@@ -19,6 +19,7 @@
  *
  */
 
+const EvView = imports.gi.EvinceView;
 const GdkPixbuf = imports.gi.GdkPixbuf;
 const Gio = imports.gi.Gio;
 const Gd = imports.gi.Gd;
@@ -53,7 +54,7 @@ const SingleItemJob = new Lang.Class({
     run: function(flags, callback) {
         this._callback = callback;
 
-        let query = Global.queryBuilder.buildSingleQuery(this._urn, flags);
+        let query = Global.queryBuilder.buildSingleQuery(flags, this._urn);
         Global.connectionQueue.add(query.sparql, null, Lang.bind(this,
             function(object, res) {
                 try {
@@ -327,8 +328,12 @@ const DocCommon = new Lang.Class({
         this.favorite = cursor.get_boolean(Query.QueryColumns.FAVORITE);
 
         let mtime = cursor.get_string(Query.QueryColumns.MTIME)[0];
-        let timeVal = Gd.time_val_from_iso8601(mtime)[1];
-        this.mtime = timeVal.tv_sec;
+        if (mtime) {
+            let timeVal = GLib.time_val_from_iso8601(mtime)[1];
+            this.mtime = timeVal.tv_sec;
+        } else {
+            this.mtime = Math.floor(GLib.get_real_time() / 1000000);
+        }
 
         this.mimeType = cursor.get_string(Query.QueryColumns.MIMETYPE)[0];
         this.rdfType = cursor.get_string(Query.QueryColumns.RDFTYPE)[0];
@@ -339,11 +344,14 @@ const DocCommon = new Lang.Class({
             this.uri = '';
 
         let title = cursor.get_string(Query.QueryColumns.TITLE)[0];
+        let filename = cursor.get_string(Query.QueryColumns.FILENAME)[0];
+
         if (title && title != '')
             this.name = title;
+        else if (filename)
+            this.name = Gd.filename_strip_extension(filename);
         else
-            this.name = Gd.filename_strip_extension(
-                cursor.get_string(Query.QueryColumns.FILENAME)[0]);
+            this.name = '';
 
         this._sanitizeTitle();
 
@@ -597,6 +605,17 @@ const DocCommon = new Lang.Class({
             retval = '{ ?urn nie:isPartOf <' + this.id + '> }';
 
         return retval;
+    },
+
+    _finishLoad: function(document, callback, exception) {
+        let docModel = null;
+        if (exception) {
+            Global.errorHandler.addLoadError(this, exception);
+        } else {
+            docModel = EvView.DocumentModel.new_with_document(document);
+        }
+
+        callback(this, docModel, exception);
     }
 });
 Signals.addSignalMethods(DocCommon.prototype);
@@ -631,9 +650,9 @@ const LocalDocument = new Lang.Class({
             function(source, res) {
                 try {
                     let document = Gd.pdf_loader_load_uri_finish(res);
-                    callback(this, document, null);
+                    this._finishLoad(document, callback, null);
                 } catch (e) {
-                    callback(this, null, e);
+                    this._finishLoad(null, callback, e);
                 }
             }));
     },
@@ -674,14 +693,10 @@ const GoogleDocument = new Lang.Class({
         let authorizer = new Gd.GDataGoaAuthorizer({ goa_object: source.object });
         let service = new GData.DocumentsService({ authorizer: authorizer });
 
-        // HACK: GJS doesn't support introspecting GTypes, so we need to use
-        // GObject.type_from_name(); but for that to work, we need at least one
-        // instance of the GType in question to have ever been created. Ensure that
-        let temp = new GData.DocumentsText();
         service.query_single_entry_async
             (service.get_primary_authorization_domain(),
              this.identifier, null,
-             GObject.type_from_name('GDataDocumentsText'),
+             GData.DocumentsText,
              cancellable, Lang.bind(this,
                  function(object, res) {
                      let entry = null;
@@ -706,11 +721,10 @@ const GoogleDocument = new Lang.Class({
                         function(source, res) {
                             try {
                                 let document = Gd.pdf_loader_load_uri_finish(res);
-                                callback(this, document, null);
+                                this._finishLoad(document, callback, null);
                             } catch (e) {
                                 // report the outmost error only
-                                callback(this, null, exception);
-                                return;
+                                this._finishLoad(null, callback, exception);
                             }
                         }));
 
@@ -721,10 +735,10 @@ const GoogleDocument = new Lang.Class({
                     (entry, service, cancellable, Lang.bind(this,
                         function(source, res) {
                             try {
-                                let document = Gd.pdf_loader_load_gdata_entry_finish(res);
-                                callback(this, document, null);
+                                let document = Gd.pdf_loader_load_uri_finish(res);
+                                this._finishLoad(document, callback, null);
                             } catch (e) {
-                                callback(this, null, e);
+                                this._finishLoad(null, callback, e);
                             }
                         }));
             }));
@@ -846,11 +860,10 @@ const SkydriveDocument = new Lang.Class({
                         function(source, res) {
                             try {
                                 let document = Gd.pdf_loader_load_uri_finish(res);
-                                callback(this, document, null);
+                                this._finishLoad(document, callback, null);
                             } catch (e) {
                                 // report the outmost error only
-                                callback(this, null, exception);
-                                return;
+                                this._finishLoad(null, callback, exception);
                             }
                         }));
 
@@ -862,9 +875,9 @@ const SkydriveDocument = new Lang.Class({
                         function(source, res) {
                             try {
                                 let document = Gd.pdf_loader_load_zpj_entry_finish(res);
-                                callback(this, document, null);
+                                this._finishLoad(document, callback, null);
                             } catch (e) {
-                                callback(this, null, e);
+                                this._finishLoad(null, callback, e);
                             }
                         }));
             }));
@@ -990,13 +1003,19 @@ const DocumentManager = new Lang.Class({
     },
 
     setActiveItem: function(doc) {
-        if (this.parent(doc)) {
+        if (!this.parent(doc))
+            return;
 
-            if (doc && !doc.collection) {
-                let recentManager = Gtk.RecentManager.get_default();
-                recentManager.add_item(doc.uri);
-            }
+        if (!doc)
+            return;
+
+        if (doc.collection) {
+            Global.collectionManager.setActiveItem(doc);
+            return;
         }
+
+        let recentManager = Gtk.RecentManager.get_default();
+        recentManager.add_item(doc.uri);
     },
 
     getModel: function() {
@@ -1008,7 +1027,14 @@ const DocumentModel = new Lang.Class({
     Name: 'DocumentModel',
 
     _init: function() {
-        this.model = Gd.create_list_store();
+        this.model = Gtk.ListStore.new(
+            [ GObject.TYPE_STRING,
+              GObject.TYPE_STRING,
+              GObject.TYPE_STRING,
+              GObject.TYPE_STRING,
+              GdkPixbuf.Pixbuf,
+              GObject.TYPE_LONG,
+              GObject.TYPE_BOOLEAN ]);
         this.model.set_sort_column_id(Gd.MainColumns.MTIME,
                                       Gtk.SortType.DESCENDING);
     },
@@ -1019,11 +1045,10 @@ const DocumentModel = new Lang.Class({
 
     documentAdded: function(doc) {
         let iter = this.model.append();
-
-        Gd.store_set(this.model, iter,
-                     doc.id, doc.uri,
-                     doc.name, doc.author,
-                     doc.pixbuf, doc.mtime);
+        this.model.set(iter,
+            [ 0, 1, 2, 3, 4, 5 ],
+            [ doc.id, doc.uri, doc.name,
+              doc.author, doc.pixbuf, doc.mtime ]);
 
         let treePath = this.model.get_path(iter);
         let treeRowRef = Gtk.TreeRowReference.new(this.model, treePath);
@@ -1036,10 +1061,10 @@ const DocumentModel = new Lang.Class({
 
                 let objectIter = this.model.get_iter(objectPath)[1];
                 if (objectIter)
-                    Gd.store_set(this.model, iter,
-                                 doc.id, doc.uri,
-                                 doc.name, doc.author,
-                                 doc.pixbuf, doc.mtime);
+                    this.model.set(objectIter,
+                        [ 0, 1, 2, 3, 4, 5 ],
+                        [ doc.id, doc.uri, doc.name,
+                          doc.author, doc.pixbuf, doc.mtime ]);
             }));
     },
 
