@@ -28,103 +28,6 @@
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnome-desktop/gnome-desktop-thumbnail.h>
 
-/* FIXME: this is here only because gjs doesn't 
- * support GType handling/arrays yet.
- */
-/**
- * gd_create_list_store:
- * 
- * Returns: (transfer full):
- */
-GtkListStore *
-gd_create_list_store (void)
-{
-  return gtk_list_store_new (7,
-                             G_TYPE_STRING, // URN
-                             G_TYPE_STRING, // URI
-                             G_TYPE_STRING, // TITLE
-                             G_TYPE_STRING, // AUTHOR
-                             GDK_TYPE_PIXBUF, // ICON
-                             G_TYPE_LONG, // MTIME
-                             G_TYPE_BOOLEAN); // SELECTED
-}
-
-void
-gd_store_set (GtkListStore *store,
-              GtkTreeIter *iter,
-              const gchar *urn,
-              const gchar *uri,
-              const gchar *title,
-              const gchar *author,
-              GdkPixbuf *icon,
-              glong mtime)
-{
-  gtk_list_store_set (store, iter,
-                      0, urn,
-                      1, uri,
-                      2, title,
-                      3, author,
-                      4, icon,
-                      5, mtime,
-                      -1);
-}
-
-/**
- * gd_create_item_store:
- * 
- * Returns: (transfer full):
- */
-GtkListStore *
-gd_create_item_store (void)
-{
-  return gtk_list_store_new (3,
-                             G_TYPE_STRING, // ID
-                             G_TYPE_STRING, // NAME
-                             G_TYPE_STRING); // HEADING_TEXT
-}
-
-void
-gd_item_store_set (GtkListStore *store,
-                   GtkTreeIter *iter,
-                   const gchar *id,
-                   const gchar *name,
-                   const gchar *heading_text)
-{
-  gtk_list_store_set (store, iter,
-                      0, id,
-                      1, name,
-                      2, heading_text,
-                      -1);
-}
-
-/**
- * gd_create_organize_store:
- * 
- * Returns: (transfer full):
- */
-GtkListStore *
-gd_create_organize_store (void)
-{
-  return gtk_list_store_new (3,
-                             G_TYPE_STRING, // ID
-                             G_TYPE_STRING, // NAME
-                             G_TYPE_INT); // STATE
-}
-
-void
-gd_organize_store_set (GtkListStore *store,
-                       GtkTreeIter *iter,
-                       const gchar *id,
-                       const gchar *name,
-                       gint state)
-{
-  gtk_list_store_set (store, iter,
-                      0, id,
-                      1, name,
-                      2, state,
-                      -1);
-}
-
 #define ATTRIBUTES_FOR_THUMBNAIL \
   G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE"," \
   G_FILE_ATTRIBUTE_TIME_MODIFIED
@@ -374,7 +277,7 @@ gd_filename_get_extension_offset (const char *filename)
 
 /**
  * gd_filename_strip_extension:
- * @filename_with_extension:
+ * @filename_with_extension: (allow-none):
  *
  * Returns: (transfer full):
  */
@@ -470,26 +373,6 @@ gd_filename_to_rdf_type (const gchar *filename_with_extension)
   return type;
 }
 
-/**
- * gd_time_val_from_iso8601:
- * @string: (allow-none):
- * @timeval: (out):
- *
- * Returns:
- */
-gboolean
-gd_time_val_from_iso8601 (const gchar *string,
-                          GTimeVal *timeval)
-{
-  if (string == NULL)
-    {
-      g_get_current_time (timeval);
-      return TRUE;
-    }
-
-  return g_time_val_from_iso8601 (string, timeval);
-}
-
 /**
  * gd_iso8601_from_timestamp:
  * @timestamp:
@@ -766,15 +649,3 @@ gd_create_variant_from_pixbuf (GdkPixbuf *pixbuf)
                                                     g_object_ref (pixbuf)));
   return g_variant_ref_sink (variant);
 }
-
-/**
- * gd_format_int_alternative_output:
- * @intval:
- *
- * Returns: (transfer full):
- */
-gchar *
-gd_format_int_alternative_output (gint intval)
-{
-  return g_strdup_printf ("%Id", intval);
-}
