@@ -24,30 +24,6 @@
 
 #include <gtk/gtk.h>
 
-GtkListStore* gd_create_list_store (void);
-void gd_store_set (GtkListStore *store,
-                   GtkTreeIter *iter,
-                   const gchar *urn,
-                   const gchar *uri,
-                   const gchar *title,
-                   const gchar *author,
-                   GdkPixbuf *icon,
-                   glong mtime);
-
-GtkListStore* gd_create_item_store (void);
-void gd_item_store_set (GtkListStore *store,
-                        GtkTreeIter *iter,
-                        const gchar *id,
-                        const gchar *name,
-                        const gchar *heading_text);
-
-GtkListStore* gd_create_organize_store (void);
-void gd_organize_store_set (GtkListStore *store,
-                            GtkTreeIter *iter,
-                            const gchar *id,
-                            const gchar *name,
-                            gint state);
-
 void gd_queue_thumbnail_job_for_file_async (GFile *file,
                                             GAsyncReadyCallback callback,
                                             gpointer user_data);
@@ -70,8 +46,6 @@ const char *gd_filename_to_mime_type (const gchar *filename_with_extension);
 
 const char *gd_filename_to_rdf_type (const gchar *filename_with_extension);
 
-gboolean gd_time_val_from_iso8601 (const gchar *string,
-                                   GTimeVal *timeval);
 gchar *gd_iso8601_from_timestamp (gint64 timestamp);
 
 GIcon *gd_create_collection_icon (gint base_size,
@@ -83,7 +57,5 @@ void   gd_entry_focus_hack (GtkWidget *entry,
 
 GVariant *gd_create_variant_from_pixbuf (GdkPixbuf *pixbuf);
 
-gchar * gd_format_int_alternative_output (gint intval);
-
 #endif /* __GD_UTILS_H__ */
                                   
