@@ -19,10 +19,12 @@
  *
  */
 
+const Clutter = imports.gi.Clutter;
 const EvView = imports.gi.EvinceView;
 const Gd = imports.gi.Gd;
 const Gdk = imports.gi.Gdk;
 const GLib = imports.gi.GLib;
+const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
 const GtkClutter = imports.gi.GtkClutter;
 const Pango = imports.gi.Pango;
@@ -317,7 +319,10 @@ const OrganizeCollectionModel = new Lang.Class({
     Name: 'OrganizeCollectionModel',
 
     _init: function() {
-        this.model = Gd.create_organize_store();
+        this.model = Gtk.ListStore.new(
+            [ GObject.TYPE_STRING,
+              GObject.TYPE_STRING,
+              GObject.TYPE_INT ]);
         this._placeholderRef = null;
 
         this._collAddedId =
@@ -367,8 +372,9 @@ const OrganizeCollectionModel = new Lang.Class({
             if (!iter)
                 iter = this.model.append();
 
-            Gd.organize_store_set(this.model, iter,
-                                  item.id, item.name, collectionState[item.id]);
+            this.model.set(iter,
+                [ 0, 1, 2 ],
+                [ item.id, item.name, collectionState[item.id] ]);
         }
     },
 
@@ -396,8 +402,9 @@ const OrganizeCollectionModel = new Lang.Class({
         this.removePlaceholder();
 
         let iter = this.model.append();
-        Gd.organize_store_set(this.model, iter,
-                              _COLLECTION_PLACEHOLDER_ID, '', OrganizeCollectionState.ACTIVE);
+        this.model.set(iter,
+            [ 0, 1, 2 ],
+            [ _COLLECTION_PLACEHOLDER_ID, '', OrganizeCollectionState.ACTIVE ]);
 
         let placeholderPath = this.model.get_path(iter);
         if (placeholderPath != null)
@@ -711,9 +718,10 @@ Signals.addSignalMethods(SelectionController.prototype);
 const SelectionToolbar = new Lang.Class({
     Name: 'SelectionToolbar',
 
-    _init: function() {
+    _init: function(parentActor) {
         this._itemListeners = {};
         this._insideRefresh = false;
+        this._parentActor = parentActor;
 
         this.widget = new Gtk.Toolbar({ show_arrow: false,
                                         icon_size: Gtk.IconSize.LARGE_TOOLBAR });
@@ -724,6 +732,33 @@ const SelectionToolbar = new Lang.Class({
                                             opacity: 0 });
         Utils.alphaGtkWidget(this.actor.get_widget());
 
+        let widthConstraint =
+            new Clutter.BindConstraint({ source: this._parentActor,
+                                         coordinate: Clutter.BindCoordinate.WIDTH,
+                                         offset: - 300 });
+        this.actor.add_constraint(widthConstraint);
+        this.actor.connect('notify::width', Lang.bind(this,
+            function() {
+                let width = this._parentActor.width;
+                let offset = 300;
+
+                if (width > 1000)
+                    offset += (width - 1000);
+                else if (width < 600)
+                    offset -= (600 - width);
+
+                widthConstraint.offset = - offset;
+            }));
+
+        this.actor.add_constraint(
+            new Clutter.AlignConstraint({ align_axis: Clutter.AlignAxis.X_AXIS,
+                                          source: this._parentActor,
+                                          factor: 0.50 }));
+        this.actor.add_constraint(
+            new Clutter.AlignConstraint({ align_axis: Clutter.AlignAxis.Y_AXIS,
+                                          source: this._parentActor,
+                                          factor: 0.95 }));
+
         this._leftBox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
         this._leftGroup = new Gtk.ToolItem({ child: this._leftBox });
         this.widget.insert(this._leftGroup, -1);
